import json as json
import re

import pkg_resources
from pydantic import BaseModel, constr

LOCATION_TEMPLATE = "{remote_parent}/{normalized_version}/{remote_name}"

LOCATION_REGEX = re.compile(
    r"""
    /                                       # Inner directory start
    (?P<location_year>[0-9]{4})             # Year of the location directory
    -                                       # Separator between year and month
    (?P<location_month>[0-9]{2})            # Month of the location directory
    /                                       # Directory sepparator
    (?P<location_normalized_version>[0-9]{3})  # Version of the location directory
    /                                       # Inner directory end
    """,
    re.VERBOSE,
)


class ASICExtesionMap(BaseModel):
    asic_extension: constr(regex=r"^\.[a-zA-Z0-9]*$")  # type: ignore # noqa: F722
    normalized_version: constr(regex=r"^[0-9]{3}$")  # type: ignore # noqa: F722
    order: int


class ASICFileConfig(BaseModel):
    code: str
    name_pattern: str
    location_pattern: str
    description: str | None


def load_asic_file_extension_map() -> dict[str, ASICExtesionMap]:
    """Return a list of ASIC extension maps"""
    # This is a stream-like object. If you want the actual info, call
    # stream.read()
    stream = pkg_resources.resource_stream("asic", "data/ASIC_FILE_EXTENSION_MAP.jsonl")
    lines = []
    for line in stream:
        lines.append(json.loads(line))

    asic_file_extension_mapper = {
        line["asic_extension"]: ASICExtesionMap.parse_obj(line) for line in lines
    }
    return asic_file_extension_mapper


def load_asic_file_config() -> dict[str, ASICFileConfig]:
    """Return a list of ASIC file configurations"""
    # This is a stream-like object. If you want the actual info, call
    # stream.read()
    stream = pkg_resources.resource_stream("asic", "data/ASIC_FILE_CONFIG.jsonl")
    lines = []
    for line in stream:
        lines.append(json.loads(line))

    asic_file_config = {line["code"]: ASICFileConfig.parse_obj(line) for line in lines}
    return asic_file_config



import enum
import pathlib

from ..metadata import FileItemInfo
from .adem import adem_preprocess
from .aenc import aenc_preprocess
from .afac import afac_preprocess
from .dspcttos import dspcttos_preprocess
from .ldcbmr import ldcbmr_preprocess
from .pep import pep_preprocess
from .sntie import sntie_preprocess
from .tgrl import tgrl_preprocess
from .trsd import trsd_preprocess
from .trsm import trsm_preprocess

# from .undefined import undefined_preprocess


class SupportedFiles(str, enum.Enum):
    TGRL = "tgrl", tgrl_preprocess
    AENC = "aenc", aenc_preprocess
    ADEM = "adem", adem_preprocess
    TRSM = "trsm", trsm_preprocess
    LDCBMR = "ldcbmr", ldcbmr_preprocess
    PEP = "pep", pep_preprocess
    TRSD = "trsd", trsd_preprocess
    SNTIE = "sntie", sntie_preprocess
    AFAC = "afac", afac_preprocess
    DSPCTTOS = "dspcttos", dspcttos_preprocess

    def __new__(cls, code, preprocessor):
        obj = str.__new__(cls, [code])
        obj._value_ = code
        obj.preprocessor = preprocessor
        return obj

    def preprocess(self, filepath: pathlib.Path, item: FileItemInfo):
        return self.preprocessor(filepath, item)


__all__ = ["SupportedFiles"]



import logging
import pathlib

import numpy as np

# Third party imports
import pandas as pd

# Local application imports
from asic.reader import FileReader

from ..metadata import FileItemInfo

logger = logging.getLogger(__name__)

adem_format = {
    "type": "csv",
    "sep": ";",
    "encoding": "cp1252",
    "dt_fields": {},
    "dtype": {
        "CODIGO": str,
        "AGENTE": str,
        "CONTENIDO": str,
        "HORA 01": float,
        "HORA 02": float,
        "HORA 03": float,
        "HORA 04": float,
        "HORA 05": float,
        "HORA 06": float,
        "HORA 07": float,
        "HORA 08": float,
        "HORA 09": float,
        "HORA 10": float,
        "HORA 11": float,
        "HORA 12": float,
        "HORA 13": float,
        "HORA 14": float,
        "HORA 15": float,
        "HORA 16": float,
        "HORA 17": float,
        "HORA 18": float,
        "HORA 19": float,
        "HORA 20": float,
        "HORA 21": float,
        "HORA 22": float,
        "HORA 23": float,
        "HORA 24": float,
    },
}


class ADEM(FileReader):
    def __init__(self):
        return super().__init__(adem_format.copy())


def adem_preprocess(filepath: pathlib.Path, item: FileItemInfo) -> pd.DataFrame:
    """
    ADEM: es un archivo diario
    versiones: TX2, TXR, TXF
    AGENTE
    EPSC: aqui va enerbit
    CODIGO:
    DMRE: demanda regulada
    PRRE: perdidas reguladas
    """
    adem_reader = ADEM()
    total = adem_reader.read(filepath)
    total["FECHA"] = f"{item.year:04d}-{item.month:02d}-{item.day:02d}"

    filter = np.full(total.index.shape, True)
    filter = filter & (total["CODIGO"].isin(["DMRE", "PRRE"]))
    if item.agent is not None:
        filter = filter & (total["AGENTE"] == item.agent.upper())

    total = total[filter]
    total["FECHA"] = pd.to_datetime(
        total["FECHA"],
        format="%Y-%m-%d",
    )
    total = (
        total.set_index(["FECHA", "CODIGO", "AGENTE", "CONTENIDO"])
        .stack()
        .reset_index()
    )
    total = total.rename(columns={"level_4": "NOMBRE HORA", 0: "VALOR"})
    total["HORA"] = (total["NOMBRE HORA"].str.slice(start=-2)).astype(int) - 1
    total["HORA"] = pd.to_timedelta(total["HORA"], unit="h")
    total["FECHA_HORA"] = total["FECHA"] + total["HORA"]

    total = (
        total[
            ["FECHA", "NOMBRE HORA", "HORA", "FECHA_HORA", "AGENTE", "CODIGO", "VALOR"]
        ]
        .set_index(["FECHA", "NOMBRE HORA", "HORA", "FECHA_HORA", "AGENTE", "CODIGO"])
        .unstack()
        .reset_index()
    )
    cols = [
        f"{l1}_{l0}" if l1 else l0
        for (l0, l1) in zip(
            total.columns.get_level_values(0), total.columns.get_level_values(1)
        )
    ]
    total.columns = cols
    return_cols = ["FECHA_HORA", "AGENTE", "DMRE_VALOR", "PRRE_VALOR"]
    return total[return_cols]


if __name__ == "__main__":
    import pathlib

    reader = ADEM()
    path = pathlib.Path("./files/PublicoK/SIC/COMERCIA/2022-06/adem0617.TxF")
    data = reader.read(path)
    print(data.head(10))
    f_info = FileItemInfo(
        path=path,
        year=2022,
        month=6,
        day=17,
        extension=".txf",
        code="adem",
        version="004",
        agent="enbc",
    )
    prepro_data = adem_preprocess(path, f_info)
    print(prepro_data.head(10))



import logging
import pathlib

# Third party imports
import pandas as pd

# Local application imports
from asic.reader import FileReader

from ..metadata import FileItemInfo

logger = logging.getLogger(__name__)

sntie_format = {
    "type": "csv",
    "sep": ";",
    "encoding": "cp1252",
    "dt_fields": {"FECHA": {"format": "%Y-%m-%d"}},
    "dtype": {
        "FECHA": str,
        "AGENTE": str,
        "CONCEPTO": str,
        "BANDERA": str,
        "HORA 01": float,
        "HORA 02": float,
        "HORA 03": float,
        "HORA 04": float,
        "HORA 05": float,
        "HORA 06": float,
        "HORA 07": float,
        "HORA 08": float,
        "HORA 09": float,
        "HORA 10": float,
        "HORA 11": float,
        "HORA 12": float,
        "HORA 13": float,
        "HORA 14": float,
        "HORA 15": float,
        "HORA 16": float,
        "HORA 17": float,
        "HORA 18": float,
        "HORA 19": float,
        "HORA 20": float,
        "HORA 21": float,
        "HORA 22": float,
        "HORA 23": float,
        "HORA 24": float,
    },
}


class SNTIE(FileReader):
    def __init__(self):
        return super().__init__(sntie_format.copy())


def sntie_preprocess(filepath: pathlib.Path, item: FileItemInfo) -> pd.DataFrame:
    """
    SNTIE: se publica un archivo por mes.
    versiones: TXF
    AGENTE:
     EPSC: el codigo del agente ##AQUI VA ENERBIT##
    CONCEPTO:
     DSNTM
    BANDERA:
     S
    """
    sntie_reader = SNTIE()
    total = sntie_reader.read(filepath)
    total = total[
        (total["AGENTE"] == item.agent)
        & (total["CONCEPTO"] == "DSNTM")
        & (total["BANDERA"] == "S")
    ]
    total = (
        total.set_index(["FECHA", "AGENTE", "CONCEPTO", "BANDERA"])
        .stack()
        .reset_index()
    )
    total = total.rename(columns={"level_4": "NOMBRE HORA", 0: "COSTO_TIES"})
    total["HORA"] = (total["NOMBRE HORA"].str.slice(start=-2)).astype(int) - 1
    total["HORA"] = pd.to_timedelta(total["HORA"], unit="h")
    total["FECHA_HORA"] = total["FECHA"] + total["HORA"]
    ret_cols = ["FECHA_HORA", "AGENTE", "CONCEPTO", "BANDERA", "COSTO_TIES"]
    return total[ret_cols]



import datetime as dt
import logging
import typing

import pandas as pd
import pydantic
import requests

from asic import ASIC_FILE_EXTENSION_MAP

logger = logging.getLogger(__name__)

SUPPORTED_ASIC_EXTENSIONS = frozenset(
    [ext.replace(".", "") for ext in ASIC_FILE_EXTENSION_MAP.keys()]
)


class ASICVersionPublication(pydantic.BaseModel):
    month: dt.datetime
    version: pydantic.constr(regex=r"^[a-z0-9]*$")  # type: ignore # noqa: F722
    published_at: dt.datetime


USER_AGENT = (
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    "AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/80.0.3987.149 Safari/537.36"
)
ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE: dict[str, typing.Any] = {
    "url": "http://sv01.xm.com.co/gmem/Admon_Mcdo/Liquidacion/versionesliq.htm",
    "table-index": 1,
    "encoding": "cp1252",
    "month_settlement_format": r"%b %Y",
    "headers": {"User-Agent": USER_AGENT},
    "table_cols": {
        "month": "MES LIQUIDADO",
        "version": "ULTIMA VERSION LIQUIDADA",
        "published_at": "FECHA DE EMISIÓN/PUBLICACIÓN",
    },
}


def get_daily_versions() -> list[ASICVersionPublication]:
    daily_versions = []
    # TX1
    daily_versions.append(
        ASICVersionPublication(
            month=dt.datetime.combine(dt.date.today(), dt.datetime.min.time())
            + dt.timedelta(days=-1),
            version="tx1",
            published_at=dt.datetime.combine(dt.date.today(), dt.datetime.min.time()),
        )
    )
    # TX2
    daily_versions.append(
        ASICVersionPublication(
            month=dt.datetime.combine(dt.date.today(), dt.datetime.min.time())
            + dt.timedelta(days=-3),
            version="tx2",
            published_at=dt.datetime.combine(dt.date.today(), dt.datetime.min.time()),
        )
    )
    return daily_versions


def get_monthly_pubs_table() -> pd.DataFrame:
    headers: dict[str, str] = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE["headers"]
    url: str = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE["url"]
    source_encoding: str = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE["encoding"]
    table_index: int = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE["table-index"]

    logger.debug(f"Getting content from '{url}'")
    res = requests.get(url, headers=headers)
    res.raise_for_status()

    html_text = typing.cast(str, res.content)
    logger.debug("Parsing content as HTML table")
    tables = pd.read_html(html_text, flavor="html5lib", encoding=source_encoding)
    versions_table = tables[table_index].dropna().T.drop_duplicates().T
    table_headers = versions_table.iloc[0]
    versions_table = pd.DataFrame(versions_table.values[1:], columns=table_headers)
    versions_table = versions_table.drop(versions_table.index[0])
    return versions_table


def prepare_published_versions_to_objects(
    versions_table: pd.DataFrame,
) -> list[ASICVersionPublication]:
    month_settlement_format = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE[
        "month_settlement_format"
    ]
    table_cols = ASIC_MONTHLY_VERSION_PUBLICATION_SERVICE["table_cols"]

    # Make sure parsed table has expected structure
    assert list(table_cols.values()) == list(versions_table)

    versions_table[table_cols["month"]] = pd.to_datetime(
        versions_table[table_cols["month"]], format=month_settlement_format
    )

    versions_table[table_cols["published_at"]] = pd.to_datetime(
        versions_table[table_cols["published_at"]], format=r"%Y-%m-%d"
    )

    versions_table[table_cols["version"]] = versions_table[
        table_cols["version"]
    ].str.lower()

    ignored_versions = versions_table[
        ~versions_table[table_cols["version"]].isin(SUPPORTED_ASIC_EXTENSIONS)
    ].copy()

    included_versions = versions_table[
        versions_table[table_cols["version"]].isin(SUPPORTED_ASIC_EXTENSIONS)
    ].copy()

    for row in ignored_versions.iterrows():
        logger.warning(
            f"Published version NOT considered: {row[table_cols['version']]} on {row[table_cols['published_at']]}"
        )

    latest_version_pub = (
        included_versions.sort_values([table_cols["published_at"]], ascending=False)
        .groupby([table_cols["month"]])
        .first()
    ).reset_index()

    latest_version_pub[table_cols["month"]] = (
        latest_version_pub[table_cols["month"]].dt.to_period("M").dt.to_timestamp()
    )
    inv_map = {v: k for k, v in table_cols.items()}
    latest_version_pub = latest_version_pub.rename(columns=inv_map)
    latest_version_pub = latest_version_pub.to_dict(orient="records")
    latest_version_pub_objects = [
        ASICVersionPublication.parse_obj(i) for i in latest_version_pub
    ]
    logger.debug(f"Latest versions len: {len(latest_version_pub_objects)}")
    return latest_version_pub_objects


def list_latest_published_versions(
    published_after: dt.datetime | None = None, include_daily: bool = False
) -> list[ASICVersionPublication]:
    versions_table = get_monthly_pubs_table()
    versions = prepare_published_versions_to_objects(versions_table)
    if include_daily:
        daily_versions = get_daily_versions()
        versions.extend(daily_versions)
    if published_after:
        versions = [v for v in versions if (v.published_at >= published_after)]
    versions = sorted(versions, key=lambda p: (p.published_at, p.month), reverse=True)

    return versions


if __name__ == "__main__":
    published_after = dt.datetime.today() - dt.timedelta(days=90)
    print(f"Keeping settlements published after {published_after}")
    newest_versions = list_latest_published_versions(published_after, True)
    for i in newest_versions:
        print(i)



from .config import (
    LOCATION_REGEX,
    LOCATION_TEMPLATE,
    load_asic_file_config,
    load_asic_file_extension_map,
)

ASIC_FILE_CONFIG = load_asic_file_config()
ASIC_FILE_EXTENSION_MAP = load_asic_file_extension_map()

__all__ = [
    "ASIC_FILE_CONFIG",
    "ASIC_FILE_EXTENSION_MAP",
    "LOCATION_REGEX",
    "LOCATION_TEMPLATE",
]



import logging
import pathlib

import pandas as pd

# Local application imports
from asic.reader import FileReader

# Third party imports
# import pandas as pd
from ..metadata import FileItemInfo

logger = logging.getLogger(__name__)

trsm_format = {
    "type": "csv",
    "sep": ";",
    "encoding": "cp1252",
    "dt_fields": {"FECHA": {"format": "%Y-%m-%d"}},
    "dtype": {"FECHA": str, "CODIGO": str, "DESCRIPCION": str, "VALOR": float},
}


class TRSM(FileReader):
    def __init__(self):
        return super().__init__(trsm_format.copy())


def trsm_preprocess(filepath: pathlib.Path, item: FileItemInfo) -> pd.DataFrame:
    """
    TRSM: se publica un archivo por mes.
    versiones: TXR, TXF
    """
    trsm_reader = TRSM()
    total = trsm_reader.read(filepath)

    total = total[total["CODIGO"] == "MC"]
    return_cols = ["FECHA", "CODIGO", "DESCRIPCION", "VALOR"]
    return total[return_cols]



import logging
import pathlib

# Third party imports
import pandas as pd

# Local application imports
from asic.reader import FileReader

from ..metadata import FileItemInfo

logger = logging.getLogger(__name__)

ldcbmr_format = {
    "type": "csv",
    "sep": ";",
    "encoding": "cp1252",
    "sheet_name": 0,
    "engine": "openpyxl",
    "dt_fields": {
        "Fecha de Reporte de la Falla": {"format": "%Y-%m-%d"},
        "Fecha Máxima de Normalización": {"format": "%Y-%m-%d"},
        "Fecha de Solicitud de Ampliación": {"format": "%Y-%m-%d"},
    },
    "dtype": {
        "Código SIC": str,
        "Tipo de Frontera": str,
        "Equipo en Falla": str,
        "Fecha de Reporte de la Falla": str,
        "Fecha Máxima de Normalización": str,
        "Días en Falla": int,
        "Solicitó Ampliación de Plazo": str,
        "Fecha de Solicitud de Ampliación": str,
        "Requerimientos": str,
        "Agente que Reportó la Falla": str,
        "Agente Representante": str,
        "Operador de Red": str,
    },
}


class LDCBMR(FileReader):
    def __init__(self):
        return super().__init__(ldcbmr_format.copy())


def ldcbmr_preprocess(filepath: pathlib.Path, item: FileItemInfo) -> pd.DataFrame:
    """
    TRSM: se publica un archivo por mes.
    versiones: TXR, TXF
    """
    ldcbmr_reader = LDCBMR()
    total = ldcbmr_reader.read(filepath)

    total = total.set_index(["FECHA", "CODIGO"]).stack().reset_index()
    total = total.rename(columns={"level_2": "NOMBRE HORA", 0: "VALOR"})
    total["HORA"] = (total["NOMBRE HORA"].str.slice(start=-2)).astype(int) - 1
    total["HORA"] = pd.to_timedelta(total["HORA"], unit="h")
    total["FECHA_HORA"] = total["FECHA"] + total["HORA"]

    total = (
        total.set_index(["FECHA", "NOMBRE HORA", "HORA", "FECHA_HORA", "CODIGO"])
        .unstack()
        .reset_index()
    )
    cols = [
        f"{l1}_{l0}" if l1 else l0
        for (l0, l1) in zip(
            total.columns.get_level_values(0), total.columns.get_level_values(1)
        )
    ]
    total.columns = cols
    return_col = ["FECHA_HORA", "CHDC_VALOR", "VHDC_VALOR"]
    return total[return_col]



import logging
import pathlib
import re

import pydantic

from asic import ASIC_FILE_CONFIG, ASIC_FILE_EXTENSION_MAP

from .config import LOCATION_REGEX

logger = logging.getLogger(__name__)


class FileItemInfo(pydantic.BaseModel):
    path: pathlib.PurePath
    year: int
    month: int
    day: int | None
    extension: str
    code: str
    version: str
    agent: str | None

    class Config:
        arbitrary_types_allowed = True


def extract_metadata_from_remote_path(
    file_path: pathlib.PurePath, as_: str | None = None
) -> FileItemInfo:
    if as_ is None:
        logger.debug(
            "Trying file patterns to discover file code,"
            " consider passing the actual 'as_' argument"
        )
        for c, config in ASIC_FILE_CONFIG.items():
            name_pattern = config.name_pattern
            logger.debug(f"Trying file pattern {name_pattern:r} of code {c}")
            if re.match(name_pattern, str(file_path)):
                as_ = c
                break
        else:
            raise ValueError(
                f"Failed to find a file code in supported files for {file_path}"
            )
    try:
        file_config = ASIC_FILE_CONFIG[as_]
    except KeyError:
        raise ValueError(f"Unsupported file code '{as_}'")

    path_pattern = LOCATION_REGEX.pattern + file_config.name_pattern
    match = re.match(path_pattern, str(file_path))
    if match is None:
        raise ValueError(
            f"failed to extract metadata from file path {file_path} using pattern {path_pattern}"
        )
    assert match.group("location_month") == match.group("month")

    year = (match.group("location_year"),)
    month = (match.group("month"),)
    extension = (match.group("ext"),)
    code = (match.group("code"),)
    version = (ASIC_FILE_EXTENSION_MAP[match.group("ext").lower()].normalized_version,)
    try:
        day = int(match.group("day"))
    except Exception:
        day = None
    return FileItemInfo(
        path=file_path,
        year=year,
        month=month,
        day=day,
        extension=extension,
        code=code,
        version=version,
    )


def extract_metadata_from_local_path(
    file_path: pathlib.PurePath, as_: str | None = None
) -> FileItemInfo:
    if as_ is None:
        logger.debug(
            "Trying file patterns to discover file code,"
            " consider passing the actual 'as_' argument"
        )
        for c, config in ASIC_FILE_CONFIG.items():
            name_pattern = config.name_pattern
            logger.debug(f"Trying file pattern {name_pattern:r} of code {c}")
            if re.match(name_pattern, str(file_path)):
                as_ = c
                break
        else:
            raise ValueError(
                f"Failed to find a file code in supported files for {file_path}"
            )
    try:
        file_config = ASIC_FILE_CONFIG[as_]
    except KeyError:
        raise ValueError(f"Unsupported file code '{as_}'")

    path_pattern = LOCATION_REGEX.pattern + file_config.name_pattern
    match = re.match(path_pattern, str(file_path))
    if match is None:
        raise ValueError(
            f"failed to extract metadata from file path {file_path} using pattern {path_pattern}"
        )
    assert match.group("location_month") == match.group("month")

    year = (match.group("location_year"),)
    month = (match.group("month"),)
    extension = (match.group("ext"),)
    code = (match.group("code"),)
    version = (ASIC_FILE_EXTENSION_MAP[match.group("ext").lower()].normalized_version,)
    try:
        day = int(match.group("day"))
    except Exception:
        day = None
    return FileItemInfo(
        path=file_path,
        year=year,
        month=month,
        day=day,
        extension=extension,
        code=code,
        version=version,
    )


if __name__ == "__main__":
    remote_path = pathlib.PurePosixPath(
        "/UsuariosK/ENBC/Sic/COMERCIA/2022-07/aenc0703.tx2"
    )
    file_info = extract_metadata_from_remote_path(remote_path, "AENC")
